@@ -183,7 +183,7 @@
  * - All internal encoding related functions will not be built
  * - Any attempt to call fish_sound_new() with \a mode == FISH_SOUND_ENCODE
  *   will fail, returning NULL
- * - Any attempt to call fish_sound_encode() will return
+ * - Any attempt to call fish_sound_encode_*() will return
  *   FISH_SOUND_ERR_DISABLED
  * - The resulting library will not be linked against libvorbisenc
  *
@@ -325,13 +325,13 @@
  * - create a FishSound* object with mode FISH_SOUND_DECODE. fish_sound_new()
  * will return a new FishSound* object, initialised for decoding, and the
  * FishSoundInfo structure will be cleared.
- * - provide a FishSoundDecoded callback for libfishsound to call when it has
+ * - provide a FishSoundDecoded_* callback for libfishsound to call when it has
  * decoded audio.
  * - (optionally) specify whether you want to receive interleaved or
  * per-channel PCM data, using a fish_sound_set_interleave().
  * The default is for per-channel (non-interleaved) PCM.
  * - feed encoded audio data to libfishsound via fish_sound_decode().
- * libfishsound will decode the audio for you, calling the FishSoundDecoded
+ * libfishsound will decode the audio for you, calling the FishSoundDecoded_*
  * callback you provided earlier each time it has a block of audio ready.
  * - when finished, call fish_sound_delete().
  *
@@ -360,10 +360,7 @@
  * encoding.
  * - provide a FishSoundEncoded callback for libfishsound to call when it
  * has a block of encoded audio
- * - (optionally) specify whether you will be providing interleaved or
- * per-channel PCM data, using a fish_sound_set_interleave().
- * The default is for per-channel (non-interleaved) PCM.
- * - feed raw PCM audio data to libfishsound via fish_sound_encode().
+ * - feed raw PCM audio data to libfishsound via fish_sound_encode_*().
  * libfishsound will encode the audio for you, calling the FishSoundEncoded
  * callback you provided earlier each time it has a block of encoded audio
  * ready.
@@ -416,34 +413,6 @@ typedef struct {
  */
 typedef void * FishSound;
 
-/**
- * Signature of a callback for libfishsound to call when it has decoded
- * audio PCM data.
- * \param fsound The FishSound* handle
- * \param pcm The decoded audio
- * \param frames The count of frames decoded
- * \param user_data Arbitrary user data
- * \retval 0 to continue
- * \retval non-zero to stop decoding immediately and
- * return control to the fish_sound_decode() caller
- */
-typedef int (*FishSoundDecoded) (FishSound * fsound, float ** pcm,
-				 long frames, void * user_data);
-
-/**
- * Signature of a callback for libfishsound to call when it has encoded
- * data.
- * \param fsound The FishSound* handle
- * \param buf The encoded data
- * \param bytes The count of bytes encoded
- * \param user_data Arbitrary user data
- * \retval 0 to continue
- * \retval non-zero to stop encoding immediately and
- * return control to the fish_sound_encode() caller
- */
-typedef int (*FishSoundEncoded) (FishSound * fsound, unsigned char * buf,
-				 long bytes, void * user_data);
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -475,50 +444,6 @@ fish_sound_identify (unsigned char * buf, long bytes);
  */
 FishSound * fish_sound_new (int mode, FishSoundInfo * fsinfo);
 
-/**
- * Set the callback for libfishsound to call when it has a block of
- * decoded audio ready
- * \param fsound A FishSound* handle (created with mode FISH_SOUND_DECODE)
- * \param decoded The callback to call
- * \param user_data Arbitrary user data to pass to the callback
- * \returns 0 on success, -1 on failure
- */
-int fish_sound_set_decoded_callback (FishSound * fsound,
-				     FishSoundDecoded decoded,
-				     void * user_data);
-
-/**
- * Set the callback for libfishsound to call when it has a block of
- * encoded data ready
- * \param fsound A FishSound* handle (created with mode FISH_SOUND_ENCODE)
- * \param encoded The callback to call
- * \param user_data Arbitrary user data to pass to the callback
- * \returns 0 on success, -1 on failure
- */
-int fish_sound_set_encoded_callback (FishSound * fsound,
-				     FishSoundEncoded encoded,
-				     void * user_data);
-
-/**
- * Decode a block of data
- * \param fsound A FishSound* handle (created with mode FISH_SOUND_DECODE)
- * \param buf A buffer of data
- * \param bytes A count of bytes to decode (ie. the length of buf)
- * \returns The number of bytes consumed
- */
-long fish_sound_decode (FishSound * fsound, unsigned char * buf, long bytes);
-
-/**
- * Encode a block of audio
- * \param fsound A FishSound* handle (created with mode FISH_SOUND_ENCODE)
- * \param pcm The audio data to encode
- * \param frames A count of frames to encode
- * \returns The number of frames encoded
- * \note For multichannel audio, the audio data is interpreted according
- * to the current PCM style
- */
-long fish_sound_encode (FishSound * fsound, float ** pcm, long frames);
-
 /**
  * Flush any internally buffered data, forcing encode
  * \param fsound A FishSound* handle
@@ -564,17 +489,6 @@ int fish_sound_command (FishSound * fsound, int command, void * data,
  */
 int fish_sound_get_interleave (FishSound * fsound);
 
-/**
- * Set the PCM format used by a FishSound object. The default value is
- * non-interleaved.
- * \param fsound A FishSound* handle
- * \param interleave Whether to use interleaved PCM or not. Valid values are
- * 0 for non-interleaved, and 1 for interleaved.
- * \retval 0 Success
- * \retval -1 Invalid \a fsound
- */
-int fish_sound_set_interleave (FishSound * fsound, int interleave);
-
 /**
  * Query the current frame number of a FishSound object.
  *
@@ -623,7 +537,7 @@ int fish_sound_set_frameno (FishSound * fsound, long frameno);
  * When encoding into Ogg, you should call this function with the \a granulepos
  * and \a eos that will be used for the \a ogg_packet structure. This call
  * should be made before passing the block of audio data to
- * fish_sound_encode(). Failure to do so may result in minor encoding errors
+ * fish_sound_encode_*(). Failure to do so may result in minor encoding errors
  * on the first and/or last packet of the stream.
  *
  * \param fsound A FishSound* handle
@@ -646,6 +560,10 @@ int fish_sound_prepare_truncation (FishSound * fsound, long next_granulepos,
 }
 #endif
 
+#include <fishsound/decode.h>
+#include <fishsound/encode.h>
 #include <fishsound/comments.h>
 
+#include <fishsound/deprecated.h>
+
 #endif /* __FISH_SOUND_H__ */
