@@ -39,15 +39,25 @@
 #include <ctype.h>
 
 #include "private.h"
+#include "convert.h"
 
 /*#define DEBUG*/
 
 #if HAVE_SPEEX
 
+#if HAVE_SPEEX_1_1
+#include <speex/speex.h>
+#include <speex/speex_header.h>
+#include <speex/speex_stereo.h>
+#include <speex/speex_callbacks.h>
+
+#else /* Speex 1.0 */
+
 #include <speex.h>
 #include <speex_header.h>
 #include <speex_stereo.h>
 #include <speex_callbacks.h>
+#endif
 
 /* Format for the vendor string: "Encoded with Speex VERSION", where VERSION
  * is the libspeex version as read from a newly-generated Speex header.
@@ -228,6 +238,41 @@ process_header(unsigned char * buf, long bytes, int enh_enabled,
   return st;
 }
 
+static int
+fs_speex_free_buffers (FishSound * fsound)
+{
+  FishSoundSpeexInfo * fss = (FishSoundSpeexInfo *)fsound->codec_data;
+
+  if (fsound->mode == FISH_SOUND_DECODE) {
+    if (fss->ipcm && fss->ipcm != fss->pcm[0]) fs_free (fss->ipcm);
+    if (fss->pcm[0]) fs_free (fss->pcm[0]);
+    if (fss->pcm[1]) fs_free (fss->pcm[1]);
+  } else {
+    if (fss->ipcm) fs_free (fss->ipcm);
+  }
+
+  return 0;
+}
+
+static inline int
+fs_speex_float_dispatch (FishSound * fsound)
+{
+  FishSoundSpeexInfo * fss = (FishSoundSpeexInfo *)fsound->codec_data;
+  FishSoundDecoded_FloatIlv df;
+  FishSoundDecoded_Float dfi;
+  int retval;
+
+  if (fsound->interleave) {
+    dfi = (FishSoundDecoded_FloatIlv)fsound->callback.decoded_float_ilv;
+    retval = dfi (fsound, (float **)fss->ipcm, fss->frame_size,
+                  fsound->user_data);
+  } else {
+    df = (FishSoundDecoded_Float)fsound->callback.decoded_float;
+    retval = df (fsound, fss->pcm, fss->frame_size, fsound->user_data);
+  }
+  
+  return retval;
+}
 
 static long
 fs_speex_decode (FishSound * fsound, unsigned char * buf, long bytes)
@@ -238,7 +283,6 @@ fs_speex_decode (FishSound * fsound, unsigned char * buf, long bytes)
   int channels = -1;
   int forceMode = -1;
   int i, j;
-  float ** retpcm;
 
   if (fss->packetno == 0) {
     fss->st = process_header (buf, bytes, enh_enabled,
@@ -297,20 +341,9 @@ fs_speex_decode (FishSound * fsound, unsigned char * buf, long bytes)
 	}
       }
 
-      if (fsound->interleave) {
-	retpcm = (float **)fss->ipcm;
-      } else {
-	retpcm = (float **)fss->pcm;
-      }
-
       fsound->frameno += fss->frame_size;
 
-      /* fss->pcm is ready to go! */
-      if (fsound->callback) {
-	((FishSoundDecoded)fsound->callback) (fsound, retpcm,
-					      fss->frame_size,
-					      fsound->user_data);
-      }
+      fs_speex_float_dispatch (fsound);
     }
   }
 
@@ -334,6 +367,7 @@ fs_speex_enc_headers (FishSound * fsound)
   SpeexHeader header;
   unsigned char * buf;
   int bytes;
+  size_t buflen;
 
   /* XXX: set wb, nb, uwb modes */
   /* These modes are declared const in speex 1.1.x, hence the explicit cast */
@@ -346,8 +380,8 @@ fs_speex_enc_headers (FishSound * fsound)
 
   fss->st = speex_encoder_init (mode);
 
-  if (fsound->callback) {
-    FishSoundEncoded encoded = (FishSoundEncoded)fsound->callback;
+  if (fsound->callback.encoded) {
+    FishSoundEncoded encoded = (FishSoundEncoded)fsound->callback.encoded;
     char vendor_string[128];
 
     /* header */
@@ -378,8 +412,9 @@ fs_speex_enc_headers (FishSound * fsound)
 
   /* XXX: blah blah blah ... set VBR etc. */
 
-  fss->ipcm = fs_malloc (fss->frame_size * fsound->info.channels
-			 * sizeof (float));
+  buflen = fss->frame_size * fsound->info.channels * sizeof (float);
+  fss->ipcm = fs_malloc (buflen);
+  memset (fss->ipcm, 0, buflen);
 
   return fsound;
 }
@@ -391,11 +426,12 @@ fs_speex_encode_write (FishSound * fsound)
   FishSoundSpeexEnc * fse = (FishSoundSpeexEnc *)fss->enc;
   int bytes;
 
+  speex_bits_insert_terminator (&fss->bits);
   bytes = speex_bits_write (&fss->bits, fse->cbits, MAX_FRAME_BYTES);
   speex_bits_reset (&fss->bits);
 
-  if (fsound->callback) {
-    FishSoundEncoded encoded = (FishSoundEncoded)fsound->callback;
+  if (fsound->callback.encoded) {
+    FishSoundEncoded encoded = (FishSoundEncoded)fsound->callback.encoded;
 
     encoded (fsound, (unsigned char *)fse->cbits, (long)bytes,
 	     fsound->user_data);
@@ -429,7 +465,7 @@ fs_speex_encode_block (FishSound * fsound)
 }
 
 static long
-fs_speex_encode_i (FishSound * fsound, float ** pcm, long frames)
+fs_speex_encode_f_ilv (FishSound * fsound, float ** pcm, long frames)
 {
   FishSoundSpeexInfo * fss = (FishSoundSpeexInfo *)fsound->codec_data;
   FishSoundSpeexEnc * fse = (FishSoundSpeexEnc *)fss->enc;
@@ -463,7 +499,7 @@ fs_speex_encode_i (FishSound * fsound, float ** pcm, long frames)
 }
 
 static long
-fs_speex_encode_n (FishSound * fsound, float * pcm[], long frames)
+fs_speex_encode_f (FishSound * fsound, float * pcm[], long frames)
 {
   FishSoundSpeexInfo * fss = (FishSoundSpeexInfo *)fsound->codec_data;
   FishSoundSpeexEnc * fse = (FishSoundSpeexEnc *)fss->enc;
@@ -516,6 +552,12 @@ fs_speex_flush (FishSound * fsound)
     nencoded += fs_speex_encode_block (fsound);
   }
 
+  /* If, at this point, fse->frame_offset == 0, then either:
+     - all remaining encoded data has just been flushed out via
+     fs_speex_encode_block(), OR
+     - there was no data remaining to flush at the beginning of this
+     function (fse->pcm_offset == 0 && fse->frame_offset == 0)
+  */
   if (fse->frame_offset == 0) return 0;
 
   while (fse->frame_offset < fss->nframes) {
@@ -531,8 +573,8 @@ fs_speex_flush (FishSound * fsound)
 
 #else /* !FS_ENCODE */
 
-#define fs_speex_encode_i NULL
-#define fs_speex_encode_n NULL
+#define fs_speex_encode_f NULL
+#define fs_speex_encode_f_ilv NULL
 #define fs_speex_flush NULL
 
 #endif
@@ -545,6 +587,35 @@ fs_speex_reset (FishSound * fsound)
   return 0;
 }
 
+static void
+fs_speex_update (FishSound * fsound, int interleave)
+{
+  FishSoundSpeexInfo * fss = (FishSoundSpeexInfo *)fsound->codec_data;
+  size_t pcm_size = sizeof (float);
+
+  fss->ipcm = (float *)
+    fs_realloc (fss->ipcm,
+		pcm_size * fss->frame_size * fsound->info.channels);
+
+  if (interleave) {
+    /* if transitioning from non-interleave to interleave,
+       free non-ilv buffers */
+    if (!fsound->interleave && fsound->info.channels == 2) {
+      if (fss->pcm[0]) fs_free (fss->pcm[0]);
+      if (fss->pcm[1]) fs_free (fss->pcm[1]);
+      fss->pcm[0] = NULL;
+      fss->pcm[1] = NULL;
+    }
+  } else {
+    if (fsound->info.channels == 1) {
+      fss->pcm[0] = (float *) fss->ipcm;
+    } else if (fsound->info.channels == 2) {
+      fss->pcm[0] = fs_realloc (fss->pcm[0], pcm_size * fss->frame_size);
+      fss->pcm[1] = fs_realloc (fss->pcm[1], pcm_size * fss->frame_size);
+    }
+  }
+}
+
 static FishSound *
 fs_speex_enc_init (FishSound * fsound)
 {
@@ -598,10 +669,13 @@ fs_speex_delete (FishSound * fsound)
 {
   FishSoundSpeexInfo * fss = (FishSoundSpeexInfo *)fsound->codec_data;
 
+  fs_speex_free_buffers (fsound);
+
   if (fsound->mode == FISH_SOUND_DECODE) {
     if (fss->st) speex_decoder_destroy (fss->st);
   } else if (fsound->mode == FISH_SOUND_ENCODE) {
     if (fss->st) speex_encoder_destroy (fss->st);
+    if (fss->enc) fs_free (fss->enc);
   }
   speex_bits_destroy (&fss->bits);
 
@@ -625,10 +699,11 @@ fish_sound_speex_codec (void)
   codec->init = fs_speex_init;
   codec->del = fs_speex_delete;
   codec->reset = fs_speex_reset;
+  codec->update = fs_speex_update;
   codec->command = fs_speex_command;
   codec->decode = fs_speex_decode;
-  codec->encode_i = fs_speex_encode_i;
-  codec->encode_n = fs_speex_encode_n;
+  codec->encode_f = fs_speex_encode_f;
+  codec->encode_f_ilv = fs_speex_encode_f_ilv;
   codec->flush = fs_speex_flush;
 
   return codec;
